# 开发人员： Xiaoqiang
# 微信公众号: xiaoqiangclub
# 开发时间： 2024/10/27 6:13
# 文件名称： file_utils.py
# 项目描述： 常用文件的读写删工具
# 开发工具： PyCharm

import os
import json
import yaml
import aiofiles
from typing import Union
from xiaoqiangclub.config.log_config import log


class FileFormatError(Exception):
    """自定义异常，表示文件格式不支持"""
    pass


def read_file(file_path: str, mode: str = 'r', encoding: str = 'utf-8') -> Union[dict, str, None]:
    """
    读取文件内容

    :param file_path: 文件路径
    :param mode: 读取模式，支持 'r' 或 'rb'
    :param encoding: 文件编码，默认为 'utf-8'
    :return: 文件内容，格式根据文件类型返回不同类型
    """
    try:
        if mode == 'r':
            with open(file_path, 'r', encoding=encoding) as file:
                if file_path.endswith('.json'):
                    return json.load(file)
                elif file_path.endswith(('.yaml', '.yml')):
                    return yaml.safe_load(file)
                elif file_path.endswith('.txt'):
                    return file.read()
                else:
                    raise FileFormatError(f"不支持的文件格式: {file_path}")
        elif mode == 'rb':
            with open(file_path, 'rb') as file:
                return file.read()
        else:
            log.error(f"不支持的读取模式: {mode}")
            return None
    except FileFormatError as e:
        log.error(str(e))
        raise
    except Exception as e:
        log.error(f"读取文件 {file_path} 时出错: {e}")
        return None


def write_file(file_path: str, content: Union[dict, str], mode: str = 'w', encoding: str = 'utf-8'):
    """
    写入内容到文件

    :param file_path: 文件路径
    :param content: 要写入的内容，支持字符串或字典
    :param mode: 写入模式，支持 'w' 或 'wb'
    :param encoding: 文件编码，默认为 'utf-8'
    """
    try:
        if mode == 'w':
            with open(file_path, 'w', encoding=encoding) as file:
                if file_path.endswith('.json'):
                    json.dump(content, file, ensure_ascii=False, indent=4)
                elif file_path.endswith(('.yaml', '.yml')):
                    yaml.dump(content, file, allow_unicode=True)
                elif file_path.endswith('.txt'):
                    file.write(content)
                else:
                    raise FileFormatError(f"不支持的文件格式: {file_path}")
        elif mode == 'wb':
            with open(file_path, 'wb') as file:
                file.write(content)
        else:
            log.error(f"不支持的写入模式: {mode}")
    except FileFormatError as e:
        log.error(str(e))
        raise
    except Exception as e:
        log.error(f"写入文件 {file_path} 时出错: {e}")


async def read_file_async(file_path: str, mode: str = 'r', encoding: str = 'utf-8') -> Union[dict, str, None]:
    """
    异步读取文件内容

    :param file_path: 文件路径
    :param mode: 读取模式，支持 'r' 或 'rb'
    :param encoding: 文件编码，默认为 'utf-8'
    :return: 文件内容，格式根据文件类型返回不同类型
    """
    try:
        if mode == 'r':
            async with aiofiles.open(file_path, mode='r', encoding=encoding) as file:
                if file_path.endswith('.json'):
                    content = await file.read()
                    return json.loads(content)
                elif file_path.endswith(('.yaml', '.yml')):
                    content = await file.read()
                    return yaml.safe_load(content)
                elif file_path.endswith('.txt'):
                    return await file.read()
                else:
                    raise FileFormatError(f"不支持的文件格式: {file_path}")
        elif mode == 'rb':
            async with aiofiles.open(file_path, mode='rb') as file:
                return await file.read()
        else:
            log.error(f"不支持的读取模式: {mode}")
            return None
    except FileFormatError as e:
        log.error(str(e))
        raise
    except Exception as e:
        log.error(f"读取文件 {file_path} 时出错: {e}")
        return None


async def write_file_async(file_path: str, content: Union[dict, str], mode: str = 'w', encoding: str = 'utf-8'):
    """
    异步写入内容到文件

    :param file_path: 文件路径
    :param content: 要写入的内容，支持字符串或字典
    :param mode: 写入模式，支持 'w' 或 'wb'
    :param encoding: 文件编码，默认为 'utf-8'
    """
    try:
        if mode == 'w':
            async with aiofiles.open(file_path, mode='w', encoding=encoding) as file:
                if file_path.endswith('.json'):
                    await file.write(json.dumps(content, ensure_ascii=False, indent=4))
                elif file_path.endswith(('.yaml', '.yml')):
                    await file.write(yaml.dump(content, allow_unicode=True))
                elif file_path.endswith('.txt'):
                    await file.write(content)
                else:
                    raise FileFormatError(f"不支持的文件格式: {file_path}")
        elif mode == 'wb':
            async with aiofiles.open(file_path, mode='wb') as file:
                await file.write(content)
        else:
            log.error(f"不支持的写入模式: {mode}")
    except FileFormatError as e:
        log.error(str(e))
        raise
    except Exception as e:
        log.error(f"写入文件 {file_path} 时出错: {e}")


def delete_file(file_path: str):
    """
    删除指定文件

    :param file_path: 文件路径
    """
    try:
        os.remove(file_path)
        log.info(f"成功删除文件: {file_path}")
    except Exception as e:
        log.error(f"删除文件 {file_path} 时出错: {e}")
